import os
import Experiment.Simulations as Simulations

class ExperimentManager():
    
    # Input: ExperimentFolder, a folder name; the ExperimentManager will store all results here
    # Process: The experiment manager checks to see if the folder exists or not, if it doesn't it creates it.
    #          The experiment manager also ensures that the folder is empty
    # Output: A new folder is created, and the path to the folder is stored as a global variable
    def __init__(self,ExperimentFolder):
        ExperimentFolder = (__file__[:__file__.rfind("\\")]+"\\"+ExperimentFolder)
        if os.path.exists(ExperimentFolder) == False:
            os.mkdir(ExperimentFolder)
            
        if len(os.listdir(ExperimentFolder)) != 0:
            input("The following directory is not empty: "+ExperimentFolder+". Are you sure you want to add more experiments to it?")
        
        self.ExperimentFolder = ExperimentFolder
    
    # Input: InputDomainSize is the desired size of the input domain. 
    #        If Experiment_Type is set to ConstantInputDomainSize, then all simulations will be based on functions that have the specified InputDomainSize
    #        Otherwise, if Experiment_Type is RandomInputDomainSize, then a given simulation's function will have an input domain size between 1 and InputDomainSize inclusive. 
    #        MaxInverseImageSize is the maximum number of inputs that can map on to the same output. It is used to randomly partition the input domain.   
    #        NumberOfSimulations is simply the number of simulations that will be used in the experiment.
    def runExperiment(self,InputDomainSize,MaxInverseImageSize,NumberOfSimulations,Experiment_Type):
        print("Running Experiment " + str(len(os.listdir(self.ExperimentFolder))+1))
        ExperimentFolder = self.ExperimentFolder + "\\Experiment_" + str(len(os.listdir(self.ExperimentFolder))+1) + "(" + str(InputDomainSize) + "," + str(MaxInverseImageSize) + "," + str(NumberOfSimulations) + "," + Experiment_Type +")"                                       
        if os.path.exists(ExperimentFolder):
            input("The following directory exists: "+ExperimentFolder+". Are you sure that you want to continue?")
        os.mkdir(ExperimentFolder)        
        Simulations.Simulations().Simulations(InputDomainSize,MaxInverseImageSize,NumberOfSimulations,ExperimentFolder,True, Experiment_Type)
        