import Experiment.Simulation as Simulation
from scipy.stats import pearsonr

class Simulations:
    
    # Input: NumberOfSimulations is the number of simulations that we want to run, and InputDomainSize,MaxInverseImageSize is how
    #        we want to parameterised the simulations
    # Process: Runs NumberOfSimulations number of simulations and stores the results of the simulations in four lists, as well as
    #          simulation data in a 5th list. A statistical test, (Pearson's correlation coefficient, by default, is then
    #          computed to compare the Probability of Collisions to all of the other data sets (apart from Dataset_SimulationsData
    # Output: The simulations data
    # I've made several additions to this method. In particular, ExperimentFolder, and StoreData are now used to decide whether to store data, and where. Additionally, the Experiment Type param,
    # determines if all simulations have the same, specified, InputDomainSize, or if the specified InputDomainSize is to be treated as an upper bound (i.e. a simulation's input domain size is 
    # between 1 and InputDomainSize).
    # I have also removed the Dataset_SimulationData and storage of this, to reduce the impact on Python's memory usage.
    def Simulations(self,InputDomainSize,MaxInverseImageSize,NumberOfSimulations,ExperimentFolder="N/A",StoreData=False,Experiment_Type="ConstantInputDomainSize",StatisticalTest=pearsonr):

        Dataset_Squeeziness = []
        Dataset_DomainToRangeRatio = []
        Dataset_ProbabilityOfCollisions = []
        Dataset_NormalisedSqueeziness = []
#         Dataset_SimulationData = []
        
        Simulator = Simulation.Simulation()
        for SimulationNumber in range(NumberOfSimulations):
            print("Running Simulation "+ str(SimulationNumber))
            SimulationData = Simulator.runSimulation(InputDomainSize,MaxInverseImageSize,ExperimentFolder,StoreData,Experiment_Type)
            Dataset_Squeeziness.append(SimulationData.Squeeziness)
            Dataset_DomainToRangeRatio.append(SimulationData.DomainToRangeRatio)
            Dataset_ProbabilityOfCollisions.append(SimulationData.ProbabilityOfCollisions)
            Dataset_NormalisedSqueeziness.append(SimulationData.NormalisedSqueeziness)
#             Dataset_SimulationData.append(SimulationData)

        
        
        SimuData = SimulationsData(
               StatisticalTest(Dataset_ProbabilityOfCollisions,Dataset_DomainToRangeRatio),
               StatisticalTest(Dataset_ProbabilityOfCollisions,Dataset_Squeeziness),
               StatisticalTest(Dataset_ProbabilityOfCollisions,Dataset_NormalisedSqueeziness),
               Dataset_Squeeziness,
               Dataset_DomainToRangeRatio,
               Dataset_ProbabilityOfCollisions,
               Dataset_NormalisedSqueeziness
#                , Dataset_SimulationData    
        )
        
        if StoreData == True:
            SimuData.saveSimulationData_CSV(ExperimentFolder)
        
        return SimuData 
        
        

# This is a data object for a single set of simulation's data
class SimulationsData:
    
    def __init__(self,PColl_vs_DRR,PColl_vs_Sq,PColl_vs_NSq,Dataset_Squeeziness,Dataset_DomainToRangeRatio,Dataset_ProbabilityOfCollisions,Dataset_NormalisedSqueeziness):
        self.PColl_vs_DRR = PColl_vs_DRR
        self.PColl_vs_Sq = PColl_vs_Sq
        self.PColl_vs_NSq = PColl_vs_NSq
        self.Dataset_Squeeziness = Dataset_Squeeziness
        self.Dataset_DomainToRangeRatio = Dataset_DomainToRangeRatio
        self.Dataset_ProbabilityOfCollisions = Dataset_ProbabilityOfCollisions
        self.Dataset_NormalisedSqueeziness = Dataset_NormalisedSqueeziness
        
    # This method is a simple interface for the saveSimulationData_CSV_RawData and saveSimulationData_CSV_ResultsData methods  
    def saveSimulationData_CSV(self,ExperimentFolder):
        self.saveSimulationData_CSV_RawData(ExperimentFolder)
        self.saveSimulationData_CSV_ResultsData(ExperimentFolder)
    
    # This method stores the four datasets as a CSV file at the specified folder
    def saveSimulationData_CSV_RawData(self,ExperimentFolder):           
        RawDataTableContents = "Simulation Number \t Squeeziness \t Domain to Range Ratio \t Probability of Collisions \t Normalised Squeeziness \n"
        for Row in range(len(self.Dataset_Squeeziness)):
            SimulationNumber = str(Row+1)
            Sq = self.Dataset_Squeeziness[Row]
            DRR = self.Dataset_DomainToRangeRatio[Row]
            PColl = self.Dataset_ProbabilityOfCollisions[Row]
            NSq = self.Dataset_NormalisedSqueeziness[Row]
            RowContents = str(SimulationNumber) + "\t" + str(Sq) + "\t" + str(DRR) + "\t" + str(PColl) + "\t" + str(NSq) + "\n"
            RawDataTableContents = RawDataTableContents + RowContents
        
        RawDataObject = None    
        try: 
            RawDataObject = open(ExperimentFolder + "\\RawData.csv","w+")
            RawDataObject.write(RawDataTableContents)
        except:
            raise Exception("Unable to save raw data.")
        finally:
            if RawDataObject != None:
                RawDataObject.close()
    
    # This method stores the statistical test results as a CSV file at the specified folder    
    def saveSimulationData_CSV_ResultsData(self,ExperimentFolder):           
        ResultsDataTableContents = "Measure 1 \t Measure 2 \t Pearson's Correlation Coefficient \t 2 Tailed P-value \n"       
        ResultsDataTableContents = ResultsDataTableContents + "Probability of Collisions \t Domain to Range Ratio \t " + str(self.PColl_vs_DRR[0]) + "\t" + str(self.PColl_vs_DRR[1]) + "\n"  
        ResultsDataTableContents = ResultsDataTableContents + "Probability of Collisions \t Squeeziness \t" + str(self.PColl_vs_Sq[0]) + "\t" + str(self.PColl_vs_Sq[1]) + "\n" 
        ResultsDataTableContents = ResultsDataTableContents + "Probability of Collisions \t Normalised Squeeziness \t" + str(self.PColl_vs_NSq[0]) + "\t" +str(self.PColl_vs_NSq[1]) + "\n"
            
        ResultsDataObject = None    
        try: 
            ResultsDataObject = open(ExperimentFolder + "\\ResultsData.csv","w+")
            ResultsDataObject.write(ResultsDataTableContents)
        except:
            raise Exception("Unable to save results data.")
        finally:
            if ResultsDataObject != None:
                ResultsDataObject.close()


        
        