import Function.InputDomain as InputDomain
import Function.OutputDomain as OutputDomain

class Function():
    
    # Input: Function is a dictionary, such that the keys for this dictionary are inputs for the function, and the associated values are the 
    #        outputs that are produced in response to these inputs
    #        ProbabilityMassFunction is a dictionary, such that the keys are input values for the function, and the values are the probabilities
    #        that these inputs will be randomly selected.
    def __init__(self,Function,ProbabilityMassFunction):
        self.InputDomain = InputDomain.InputDomain(ProbabilityMassFunction)                
        self.OutputDomain = OutputDomain.OutputDomain(ProbabilityMassFunction,Function)                
    
    # Computes Squeeziness, by means of Method One
    def compute_Squeeziness_MethodOne(self):
        return (self.InputDomain.compute_Entropy())-(self.OutputDomain.compute_Entropy())
    
    # Computes the Probability of Collisions
    def compute_Probability_of_Collision(self):
        PColl = 0
        for Output in self.OutputDomain.Alphabet:
            PColl_Output = (len(self.OutputDomain.InverseImages[Output])*(len(self.OutputDomain.InverseImages[Output])-1))/(self.OutputDomain.compute_InputDomainSize()*(self.OutputDomain.compute_InputDomainSize()-1))
            PColl = PColl + PColl_Output 
        return PColl
    
    # Computes the Domain to Range Ratio
    def compute_Domain_to_Range_Ratio(self):
        return len(self.InputDomain.Alphabet)/len(self.OutputDomain.Alphabet)
    
    # Computes Normalised Squeeziness
    def compute_Normalised_Squeeziness(self):
        return self.compute_Squeeziness_MethodOne()/self.InputDomain.compute_Entropy()